// Program Headers
#include "gameinfo.h"
#include "qtvba.h"
#include "rom_associate.h"
#include "vba_help.h"

// QT Headers
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qdatetime.h>
#include <qdir.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include <qprocess.h>
#include <qptrlist.h>
#include <qspinbox.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qprogressdialog.h>
#include <qnetwork.h>
#include <qurloperator.h>

// c++ headers
#include <iostream.h>

// images
#include "logo.xpm"
#include "paypal_logo.xpm"

qtvba::qtvba ( QWidget* parent,  const char* name, WFlags fl ) : qtvba_parent ( parent, name, fl ) {
	rom_listview->setAllColumnsShowFocus(true);
	rom_listview->setSorting(1, true);
	load_config();
	load_gameinfo();
	load_roms();
	screenshot->setPixmap(QPixmap(logo_xpm));
	screenshot2->setPixmap(QPixmap(paypal_logo_xpm));
}

void qtvba::load_roms() {
	rom_listview->clear();
	QDir romdir(path_to_roms_le->text());
	romdir.setFilter(QDir::Files | QDir::Readable);
	QStringList temp = romdir.entryList("[0-9][0-9][0-9][0-9]*.gba [0-9][0-9][0-9][0-9]*.GBA [0-9][0-9][0-9][0-9]*.bin)");
	QStringList filtered = temp.grep(filter_rom_le->text(), false);
	
	for ( QStringList::Iterator it = filtered.begin(); it != filtered.end(); ++it ) {
		QString temp_it = *it;
		QString temp_id = temp_it.left(4);
		QString temp_country, temp_name = temp_it.mid(7, temp_it.find("(")-8) , temp_lastplayed = "Never", temp_country_char = temp_it.mid(temp_it.find("(")+1, 1);
		if (temp_country_char == "U") temp_country = "USA";
		else if (temp_country_char == "E") temp_country = "Europe";
		else if (temp_country_char == "J") temp_country = "Japan";
		else if (temp_country_char == "G") temp_country = "Germany";
		else temp_country = "Unknown";

		int temp_plays = 0;
		int flag_found = 0 ;
		for (gameinfo * temp_gameinfo = games.first(); temp_gameinfo; temp_gameinfo=games.next()) {
			if (temp_gameinfo->getID() == temp_id) {
				flag_found = 1;
				temp_lastplayed = temp_gameinfo->getLastPlayed();
				temp_plays = temp_gameinfo->getPlays();
			}
		}
		if (flag_found == 0) {
			games.append(new gameinfo("","","","",temp_id, temp_lastplayed, temp_plays));
		}
		new QListViewItem(rom_listview, temp_id, temp_name, temp_country, QString::number(temp_plays), temp_lastplayed, temp_it); 
	}
	rom_count_label->setText(QString("Roms Displayed: ") + QString::number(rom_listview->childCount()));
}


void qtvba::play_rom() {
	if (rom_listview->selectedItem() != NULL) {
		QString buf;
		QString rom_to_play=rom_listview->selectedItem()->text(5);
		proc = new QProcess (this);
		proc->addArgument(path_to_vba_le->text());
		if (options_le->text() != "") {
			proc->addArgument(options_le->text());
		}
		if (!mmx_cb->isChecked()) {
			proc->addArgument("--no-mmx");
		}
		if (fullscreen_cb->isChecked()) {
			proc->addArgument("-F");
		}
		if (frameskip_cb->isChecked()) {
			proc->addArgument("--auto-frameskip");
		} else {
			proc->addArgument("--no-auto-frameskip");
		}
		if (bios_le->text() != "") {
			proc->addArgument("-b");
			proc->addArgument(bios_le->text());
		}
		if (fps_cb->isChecked()) {
			proc->addArgument("--show-speed-detailed");
		}
		
		proc->addArgument("-f");
		proc->addArgument(QString::number(filter_cb->currentItem()));
		proc->addArgument("-" + zoom_sb->cleanText());
		proc->addArgument(path_to_roms_le->text() + "/" + rom_to_play);
		connect( proc, SIGNAL(readyReadStdout()), this, SLOT(readFromStdOut()) );
		connect( proc, SIGNAL(readyReadStderr()), this, SLOT(readFromStdErr()) );
		if (!proc->start()) {
			QMessageBox::warning(this, "VisualBoyAdvance Launch Error", "There was an error executing VisualBoyAdvance", "OK");
		}
		for (gameinfo * temp_gameinfo = games.first(); temp_gameinfo; temp_gameinfo=games.next()) {
			if (temp_gameinfo->getID() == rom_listview->selectedItem()->text(0)) {
				temp_gameinfo->setLastPlayed(QDate::currentDate().toString("yyyy-MM-dd"));
				temp_gameinfo->setPlays(temp_gameinfo->getPlays() + 1);
				rom_listview->selectedItem()->setText(3, QString::number(temp_gameinfo->getPlays()) );
				rom_listview->selectedItem()->setText(4, temp_gameinfo->getLastPlayed() );
			}
		}
		// oload_roms();
	}
}

void qtvba::readFromStdOut() {
	while (proc->canReadLineStdout()) {
	 	cout << proc->readLineStdout() << endl;
	}
}

void qtvba::readFromStdErr() {
	while (proc->canReadLineStderr()) {
	 	cout << proc->readLineStderr() << endl;
	}
}

void qtvba::closeEvent (QCloseEvent *ce) {
	save_config();
	save_gameinfo();
	ce->accept();

}

void qtvba::save_config() {
	QFile f( QDir::homeDirPath() + "/.qtvba/qtvba.dat" );
	if (f.open( IO_WriteOnly )) {
		QDataStream s( &f );
		s << QString("0.2") << path_to_vba_le->text() << path_to_roms_le->text() << path_to_screenshots_le->text();
		s << options_le->text() << (int)fullscreen_cb->isChecked() << (int)mmx_cb->isChecked() << filter_cb->currentText();
		s << zoom_sb->value() << (int)frameskip_cb->isChecked() << bios_le->text() << (int)fps_cb->isChecked() << listing_le->text();
		s.unsetDevice();
		f.close();
	}
}
	
void qtvba::load_config() {
	QString version, listing; 
	QString homedir = QDir::homeDirPath();
	QDir d (homedir + "/.qtvba");
	if (!d.exists()) { (new QDir())->mkdir(homedir + "/.qtvba", true); }
	QFile f( homedir + "/.qtvba/qtvba.dat" );
	if (f.open( IO_ReadOnly )) {
		QDataStream s( &f );
		while (!s.atEnd()) {
			s >> version >> vba_path >> rom_path >> screenshot_path >> options; 
			s >> fullscreen >> mmx >> filter >> zoom >> frameskip >> bios_path >> fps >> listing;
		}
		s.unsetDevice();
		f.close();
	} else {
		QMessageBox::warning(this, "file open error", "QTVBA was not able to open it's config file, using defaults instead.", "OK");
		version = "0.2";
		vba_path = "VisualBoyAdvance";
		rom_path = homedir + "/.qtvba/roms";
		screenshot_path = homedir + "/.qtvba/screenshots";
		options="";
		fullscreen=0;
		mmx=1;
		zoom=1;
		frameskip=1;
		fps=0;
		filter="Normal";
		listing="http://www.gba-shell.com/tempory/List.txt";
	}
	if (version != "0.2") {
		QMessageBox::warning(this, "version error", "QTVBA's config file was for version " + version + " of QTVBA, using defaults instead.", "OK");
		version = "0.2";
		vba_path = "VisualBoyAdvance";
		rom_path = homedir + "/.qtvba/roms";
		screenshot_path = homedir + "/.qtvba/screenshots";
		options="";
		fullscreen=0;
		mmx=1;
		zoom=1;
		frameskip=1;
		fps=0;
		filter="Normal";
		listing="http://www.gba-shell.com/tempory/List.txt";
	}
	path_to_vba_le->setText(vba_path);
	path_to_roms_le->setText(rom_path);
	path_to_screenshots_le->setText(screenshot_path);
	options_le->setText(options);
	fullscreen_cb->setChecked(fullscreen);
	mmx_cb->setChecked(mmx);
	filter_cb->setCurrentText(filter);
	zoom_sb->setValue(zoom);
	frameskip_cb->setChecked(frameskip);
	fps_cb->setChecked(fps);
	bios_le->setText(bios_path);
	listing_le->setText(listing);
}

void qtvba::update_screenshot_and_gameinfo() {
	if (rom_listview->selectedItem() != NULL) {
		QString filename = path_to_screenshots_le->text() + "/" + rom_listview->selectedItem()->text(0) + "-1.png";
		if (QFile(filename).exists()) {
			screenshot->setPixmap(QPixmap(filename));
		} else {
			screenshot->setPixmap(QPixmap(logo_xpm));
		}	
		filename = path_to_screenshots_le->text() + "/" + rom_listview->selectedItem()->text(0) + "-2.png";
		if (QFile(filename).exists()) {
			screenshot2->setPixmap(QPixmap(filename));
		} else {
			screenshot2->setPixmap(QPixmap(paypal_logo_xpm));
		}	
	}

}

void qtvba::load_savegames() {
	QString filename = rom_listview->selectedItem()->text(5).left(rom_listview->selectedItem()->text(5).find("."));
	QDir savegamesdir(path_to_roms_le->text());
	savegamesdir.setFilter(QDir::Files | QDir::Readable);
	savegame_listbox->clear();
	QStringList savegamesstrlist(savegamesdir.entryList("*.sgm"));
	QStringList savegamesgrepstrlist(savegamesstrlist.grep(filename));
	savegame_listbox->insertStringList(savegamesgrepstrlist);
}

void qtvba::save_gameinfo() {
	gameinfo *temp;
	QFile f( QDir::homeDirPath() + "/.qtvba/gameinfo-0.2.dat" );
	if (f.open( IO_WriteOnly )) {
		QDataStream s( &f );
		for (temp = games.first(); temp; temp=games.next()) {
			s << temp->getCountry() << temp->getName() << temp->getPublisher() << temp->getYear();
			s << temp->getID() << temp->getLastPlayed() << temp->getPlays();
		}
		s.unsetDevice();
		f.close();
	}
}

void qtvba::load_gameinfo() {
	QString gameinfo_country, gameinfo_name, gameinfo_publisher, gameinfo_year, gameinfo_id, gameinfo_lastplayed;
	int gameinfo_plays = 0;
	QFile f( QDir::homeDirPath() + "/.qtvba/gameinfo-0.2.dat" );
	if (f.open( IO_ReadOnly )) {
		QDataStream s( &f );
		while (!s.atEnd()) {
			s >> gameinfo_country >> gameinfo_name >> gameinfo_publisher >> gameinfo_year;
			s >> gameinfo_id >> gameinfo_lastplayed >> gameinfo_plays;
			games.append(new gameinfo(gameinfo_country, gameinfo_name, gameinfo_publisher, gameinfo_year, gameinfo_id, gameinfo_lastplayed, gameinfo_plays));
		}
		s.unsetDevice();
		f.close();
	}
}

void qtvba::launch_rom_associate() {
	rom_associate * ra = new rom_associate(path_to_roms_le->text(), listing_le->text(), path_to_vba_le->text(), this, "ra", WType_Modal);
	ra->show();
 	connect(ra, SIGNAL(rom_renamed()), this, SLOT(load_roms()));
}

void qtvba::bios_path_browse() {
	QString bios_path = QFileDialog::getOpenFileName("/usr/share/games/VisualBoyAdvance/", "*", this, "Bios Browse");
	if (bios_path != NULL) {
		bios_le->setText(bios_path);
	}
}
void qtvba::screenshot_path_browse() {
	QString qtvba_path = QDir::homeDirPath() + QString("/.qtvba/");
	QString screenshot_path = QFileDialog::getExistingDirectory(qtvba_path, this, "get existing directory", "Locate Screenshot Directory");
	if (screenshot_path != NULL) {
		path_to_screenshots_le->setText(screenshot_path);
	}
}
void qtvba::rom_path_browse() {
	QString qtvba_path = QDir::homeDirPath() + QString("/.qtvba/");
	QString rom_path = QFileDialog::getExistingDirectory(qtvba_path, this, "get existing directory", "Locate Roms Directory");
	if (rom_path != NULL) {
		path_to_roms_le->setText(rom_path);
	}
}
void qtvba::vba_path_browse() {
	QString vba_path = QFileDialog::getOpenFileName("/usr/games/bin/VisualBoyAdvance", "*", this, "VisualBoyAdvance Locate");
	if (vba_path != NULL) {
		path_to_vba_le->setText(vba_path);
	}
}

void qtvba::dl_new_listing() {
	qInitNetworkProtocols();
	QUrlOperator *op = new QUrlOperator();
	QString destination = QDir::homeDirPath() + QString("/.qtvba/");
	QProgressDialog progress( "Downloading listing...", "Cancel", 100);
	connect(op, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * )), &progress, SLOT(setProgress(int, int)));
	op->copy( listing_le->text(), destination);
	progress.exec();
	op->stop();
}

void qtvba::show_vba_help() {
	vba_help * vbah = new vba_help(path_to_vba_le->text());
	vbah->show();
}
